from django.contrib import admin

# Register your models here.
from . import models

admin.site.register(models.BasicItem)
admin.site.register(models.PaymentProcess)
admin.site.register(models.PaymentSuccess)
admin.site.register(models.SubscriptionProcess)
admin.site.register(models.SubscriptionSuccess)
admin.site.register(models.SubscriptionPayment)







from django.db import models
from django.db.models import Q
from django.utils.translation import ugettext_lazy as _
from django.urls import reverse
from django.conf import settings

# Create your models here.

class BasicItem(models.Model):
    """
    Users should inherit from this class for modeling
    their products/items. Then users of this application
    could access their items with:

    basic_item.item_model_name.
    """

    # We need this because there are some products
    # that arent paid at all. We need logging scheme
    # for the products a given user has owned be it free
    # or not.
    user = models.ForeignKey(settings.PAYBILLS_USER, 
    null=False, on_delete=models.CASCADE, related_name='items')

    created  = models.DateTimeField(auto_now_add=True, null=True)

class BasicService(models.Model):
    """
    """

    # Users who had subscription sign up success.
    users = models.ManyToManyField(
    settings.PAYBILLS_USER, related_name='services', 
    symmetrical=False)

    created  = models.DateTimeField(auto_now_add=True, null=True)

class PaymentProcess(models.Model):
    """
    This record is created from ManualProcess view
    it yields information about the Item thats
    being purchased.
    """

    user = models.ForeignKey(settings.PAYBILLS_USER, 
    null=False, on_delete=models.CASCADE, related_name='manual_proceses')

    # A given payment process should be mapped to just
    # one item. It avoids misbehaviors.
    item = models.OneToOneField('BasicItem', null=False,
    on_delete=models.CASCADE, related_name='payment_process')

    created  = models.DateTimeField(auto_now_add=True, null=True)

class PaymentSuccess(models.Model):
    process = models.OneToOneField('PaymentProcess', 
    null=False, on_delete=models.CASCADE, related_name='success')

    created  = models.DateTimeField(auto_now_add=True, null=True)

class SubscriptionProcess(models.Model):
    """
    This table records all times an user
    starts a subscription process. 

    it is used by SubscriptionSuccess to check
    divergent information about the process.

    It is important to notice that for starting
    a subscription process it is necessary to have
    already an user account.
    """

    # An user can purchase multiple services.
    user = models.ForeignKey(settings.PAYBILLS_USER, 
    null=False, on_delete=models.CASCADE, related_name='subscription_processes')

    # An user should be subscribed just once to a service.
    service = models.OneToOneField('BasicService', 
    null=False, on_delete=models.CASCADE, related_name='subscription_processes')

    invoice_id = models.IntegerField(null=True, default=15)
    created  = models.DateTimeField(auto_now_add=True, null=True)

class SubscriptionSuccess(models.Model):
    """
    This table records all times the customer
    has subscribed and it was succesful.

    A customer may subscribe multiple times
    it should be logged here.
    """

    # A SubscriptionSuccess record is related to a 
    # SubscriptionProcess record.
    process = models.OneToOneField('SubscriptionProcess', 
    null=False, on_delete=models.CASCADE)
    created  = models.DateTimeField(auto_now_add=True, null=True)

class SubscriptionPayment(models.Model):
    """
    These records are payments that were made
    from a given subscription process.
    """

    process = models.ForeignKey('SubscriptionProcess', 
    null=False, on_delete=models.CASCADE)
    created  = models.DateTimeField(auto_now_add=True, null=True)

class UnsubscriptionSuccess(models.Model):
    process = models.OneToOneField('SubscriptionSuccess', 
    null=False, on_delete=models.CASCADE)
    created  = models.DateTimeField(auto_now_add=True, null=True)








from django.urls import reverse
from django.conf import settings
from html import escape
from . import models
import time
import hmac


class PayPalForm:
    def __init__(self, user, item, payload):
        PAYPAL_IPN_URL = '%s%s' % (settings.PAYPAL_IPN_DOMAIN, 
        reverse(settings.PAYPAL_IPN_VIEW))
        
        payload.update({
        'notify_url': PAYPAL_IPN_URL,  
        'bn': settings.PAYPAL_BUSINESS_NAME,
        'business': settings.PAYPAL_ID, })
        print('notify_url:', payload['notify_url'])
        self.payload = payload

    def __str__(self):
        param  = '<input type="hidden" name="%s" value="%s"/>'
        fmt    = lambda key, value: param % (escape(key), escape(value))
        action = (settings.PAYPAL_URL , "/cgi-bin/webscr")
        hidden = '\n'.join([fmt(ind[0], str(ind[1])) 
        for ind in self.payload.items()])

        html = ("<meta charset='utf-8' />",
        "<p>Redirecting...</p>",
        '<form method="post" action="%s%s">' % action,
         hidden, "</form>",
        "<script>document.forms[0].submit()</script>")
        return '\n'.join(html)

    def encode_id(self, id):
        key = hmac.new(settings.SECRET_KEY.encode(), 
        ('payid ' + str(id)).encode()).hexdigest()

        return settings.PAYMENTS_REALM + ' ' + str(id) + ' ' + key

class ManualForm(PayPalForm):
    def __init__(self, user, item, payload):
        record = models.PaymentProcess.objects.create(
        user=user, item=item)

        payload.update({
        'invoice': '%s-1' % self.encode_id(record.id + time.time()), 
        'custom': record.id, 
        'cmd': '_xclick'})

        super(ManualForm, self).__init__(user, item, payload)

class SubscriptionForm(PayPalForm):
    def __init__(self, user, item, payload):
        record = models.SubscriptionProcess.objects.create(
        user=user, item=item)

        payload.update({
        'invoice': '%s-1' % self.encode_id(record.id + time.time()), 
        'custom': record.id, 
        'cmd': '_xclick-subscriptions'})

        super(SubscriptionForm, self).__init__(user, item, payload)











from django import forms
import core_app.models
from . import models

# class SignupForm(forms.Form):
    # name         = forms.CharField()
    # email        = forms.EmailField()
    # password     = forms.CharField()
    # organization = forms.CharField()










from paybills.submitters import ManualForm, SubscriptionForm
from django.conf import settings
from django.views.generic import View
from django.shortcuts import render, redirect
from django.core.mail import send_mail
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt
from django.utils.decorators import method_decorator
from random import randint
from . import forms
from . import models
import requests

# Create your views here.

@method_decorator(csrf_exempt, name='dispatch')
class PayPalIPN(View):
    def post(self, request):
        print('PayPalIPN Details:%s' % request.POST)
        url = '%s/cgi-bin/webscr' % settings.PAYPAL_URL

        req = requests.post(url, 'cmd=_notify-validate&%s' % request.body.decode(
        request.POST.get('charset') or request.content_params['charset']),
        headers={'content-type': request.content_type})

        if req.text != 'VERIFIED':
            self.on_invalid_call(request.POST)
        else:
            self.on_valid_call(request.POST)
        return HttpResponse(status=200)
    
    def on_invalid_call(self, data):
        """    
        Hacker attempts/not validated ipn calls by paypal.
        """

        print('on_invalid_call:%s', data)

    def on_valid_call(self, data):
        map = {'web_accept': self.handle_web_accept,
        'recurring_payment': self.handle_recurring_payment,
        'subscr_payment': self.handle_recurring_payment,
        'subscr_cancel': self.handle_unsubscr_cancel,
        'subscr_signup': self.handle_subscr_signup}

        handle = map[data['txn_type']]
        handle(data)

    def handle_web_accept(self, data):
        """
        """

        record0 = models.PaymentProcess.objects.get(
        id=data['custom'])

        # Enable the user, it remains saving
        # information on the PaymentSuccess record.
        # I need to add more fields to it in order
        # to better log information.
        record1 = models.PaymentSuccess.objects.create(process=record0)
        self.on_manual_payment(data, record0.user, record0.item)
        self.on_success(data, record0.user, record0.item)

    def handle_recurring_payment(self, data):
        record0 = models.SubscriptionProcess.objects.get(
        id=data['custom'])

        record1, _ = models.SubscriptionSuccess.objects.get_or_create(
        process=record0)

        record2 = models.SubscriptionPayment.objects.create(process=record0)

        self.on_success(data, record0.user, record0.item)
        self.on_subscription_payment(data, record0.user, record0.item)

    def handle_subscr_signup(self, data):
        """
        """

        record0 = models.SubscriptionProcess.objects.get(
        id=data['custom'])

        record1 = models.SubscriptionSuccess.objects.get_or_create(process=record0)
        self.on_subscription_signup(data, record0.user, record0.item)

    def handle_unsubscr_cancel(self, data):
        record0 = models.SubscriptionProcess.objects.get(
        id=data['custom'])

        record1 = models.SubscriptionSuccess.objects.get(
        id=data['custom'])

        record1 = models.UnsubscriptionSuccess.objects.create(
        process=record0)

        self.on_subscription_cancel(data, record0.user, record0.item)

    def on_subscription_cancel(self, data, user, item):
        pass

    def on_subscription_signup(self, data, user, item):
        pass

    def on_manual_payment(self, data, user, item):
        pass

    def on_subscription_payment(self, data, user, item):
        pass

    def on_success(self, data, user, item):
        """
        Users should override this method to perform
        actions when payments are made.
        """
        pass








from subprocess import check_output

def get_addr(port):
    dom = 'http://%s:8000' % check_output("lynx -dump -hiddenlinks=ignore \
    -nolist http://checkip.dyndns.org:8245/ \
    | awk '{ print $4 }' | sed '/^$/d; s/^[ \
    ]*//g; s/[ ]*$//g'", shell=True).decode('utf-8').rstrip().strip()
    return dom



#! /usr/bin/env python3

from distutils.core import setup

setup(name="django-paybills",
      version="1.0.0",
      description="Django app for paypal integration.",
      packages=["paybills"],
      author="Iury O. G. Figueiredo",
      author_email="ioliveira@id.uff.br",
      url='',
      download_url='',
      keywords=[],
      classifiers=[])































































